Segment ledA = Segment(Plane(vec2( 1.0,-1.0), vec2(0.1, 0.0)),
                       Plane(vec2( 0.0,-1.0), vec2(1.0, 0.9)),
                       Plane(vec2(-1.0,-1.0), vec2(5.0, 0.9)),
                       Plane(vec2( 0.0, 1.0), vec2(5.9, 0.0)));

Segment ledB = Segment(Plane(vec2( 1.0, 0.0), vec2(0.0, 0.1)),
                       Plane(vec2(-1.0,-1.0), vec2(0.0, 5.9)),
                       Plane(vec2(-1.0, 0.0), vec2(0.9, 5.0)),
                       Plane(vec2(-1.0, 1.0), vec2(0.9, 1.0)));

Segment ledC = Segment(Plane(vec2(-1.0, 0.0), vec2(6.0, 0.1)),
                       Plane(vec2( 1.0,-1.0), vec2(6.0, 5.9)),
                       Plane(vec2( 1.0, 0.0), vec2(5.1, 5.0)),
                       Plane(vec2( 1.0, 1.0), vec2(5.1, 1.0)));

Segment ledD = Segment(Plane(vec2( 0.0,-1.0), vec2(0.1, 6.0)),
                       Plane(vec2(-1.0, 1.0), vec2(5.9, 6.0)),
                       Plane(vec2( 0.0, 1.0), vec2(5.0, 5.1)),
                       Plane(vec2( 1.0, 1.0), vec2(1.0, 5.1)));

const float scale = 10.0/120.0;

// returns true if any component of the two vec4's is non zero
bool any(vec4 a, vec4 b)
{
    a = clamp(dot(a, vec4(1.0)), 0.0, 1.0);
    b = clamp(dot(b, vec4(1.0)), 0.0, 1.0);

    return bool(max(a, b)); 
}

vec3 calcPlaneEq(Plane p, vec2 offset)
{
    return vec3(p.normal, dot(-p.normal, (p.point + offset)*scale));
}

float insideSegment(vec2 pos, Segment seg, vec2 offset)
{
    vec3 p0 = calcPlaneEq(seg.p0, offset);
    vec3 p1 = calcPlaneEq(seg.p1, offset);
    vec3 p2 = calcPlaneEq(seg.p2, offset);
    vec3 p3 = calcPlaneEq(seg.p3, offset);

    vec4 tmp = pos.x * vec4(p0.x, p1.x, p2.x, p3.x); 
    tmp = (pos.y * vec4(p0.y, p1.y, p2.y, p3.y)) + tmp;
    tmp = clamp(tmp + vec4(p0.z, p1.z, p2.z, p3.z), 0.0, 1.0);

    tmp.xy = min(tmp.xy, tmp.zw);
    return min(tmp.x, tmp.y);
}

// returns true if point is inside any segment, and that segment is on
bool insideDigit(vec2 point, float xOffset, Digit d)
{
    vec4 bottom;
    bottom.x = insideSegment(point, ledA, vec2(xOffset, 1.0));
    bottom.y = insideSegment(point, ledB, vec2(xOffset, 1.0));
    bottom.z = insideSegment(point, ledC, vec2(xOffset, 1.0));
    bottom.w = insideSegment(point, ledD, vec2(xOffset, 1.0));

    vec4 top;
    top.x = insideSegment(point, ledA, vec2(xOffset, 7.0));
    top.y = insideSegment(point, ledB, vec2(xOffset, 7.0));
    top.z = insideSegment(point, ledC, vec2(xOffset, 7.0));
    top.w = insideSegment(point, ledD, vec2(xOffset, 7.0));

    return any(bottom*d.bottom, top*d.top);
}

// returns true if point is inside any segment, even if that segment is off
bool insideDigit(vec2 point, float xOffset)
{
    vec4 bottom;
    bottom.x = insideSegment(point, ledA, vec2(xOffset, 1.0));
    bottom.y = insideSegment(point, ledB, vec2(xOffset, 1.0));
    bottom.z = insideSegment(point, ledC, vec2(xOffset, 1.0));
    bottom.w = insideSegment(point, ledD, vec2(xOffset, 1.0));

    vec4 top;
    top.x = insideSegment(point, ledA, vec2(xOffset, 7.0));
    top.y = insideSegment(point, ledB, vec2(xOffset, 7.0));
    top.z = insideSegment(point, ledC, vec2(xOffset, 7.0));
    top.w = insideSegment(point, ledD, vec2(xOffset, 7.0));

    return any(bottom, top);
}

// returns true if point is inside the specified circle
bool insideCircle(vec2 curPos, vec2 circleCenter, float radius)
{
    return (distance(curPos, circleCenter * scale) <= radius*scale);
}
